<template>
	<div style="width: calc(100ch - 28px)">
		<template v-if="data && data.length > 0">
			<template v-if="summaryList && summaryList.length > 0">
				<div class="w-full" v-for="(item, idx) in summaryList" :key="idx">
					<h3>{{ item.title }}</h3>
					<el-table ref="tableRefList" class="w-full" :data="[{}]">
						<el-table-column v-for="(col, index) in item.values" :label="col.title" :key="index">
							<template #default="scope">
								{{ col?.value }}
							</template>
						</el-table-column>
					</el-table>
				</div>
			</template>

			<div
				class="flex-column w-full"
				:class="{ 'mt-10': summaryList && summaryList.length > 0 }"
				v-if="recordSetList && recordSetList.length > 0"
			>
				<div class="min-h-[48rem] flex-auto w-full">
					<div ref="chartRefList" v-for="(item, index) in recordSetList" :key="index"></div>
				</div>
			</div>
			<div v-if="urlList && urlList.length > 0" class="w-full">
				<HTMLCom data="/test"  v-for="(item, index) in urlList" :key="index"></HTMLCom>
			</div>
		</template>
		<div v-else class="text-red-500">{{ error }}</div>
	</div>
</template>

<script setup lang="ts">
import * as echarts from 'echarts';
import type { TableInstance } from 'element-plus';
import _ from 'lodash';
import { computed, onMounted, ref } from 'vue';
import { ChartTypeEnum, chartTypeMapEchart, chartTypeMapName } from '../types';
import { axisLabelFormatter } from '/@/utils/chart';
import { timeDataOptionToContent } from '../common';
import { AnswerType } from '../../model/types';
import HTMLCom from '../htmlCom/HTMLCom.vue';
const props = defineProps(['data', 'error']);

const selectChartType = ref<ChartTypeEnum>(ChartTypeEnum.Line);

const columnsWidth = 120;

const chartRef = ref<HTMLDivElement[] | HTMLDivElement>(null);
const tableRef = ref<TableInstance>(null);
const tableRefList = computed(() => {
	let refList: TableInstance[] = [];
	if (!tableRef.value) {
		refList = [];
	} else {
		const first = tableRef.value?.[0];
		if (first) {
			refList = tableRef.value as unknown as TableInstance[];
		} else {
			refList = [tableRef.value as unknown as TableInstance];
		}
	}

	return refList;
});

const chartRefList = computed(() => {
	let refList: HTMLDivElement[] = [];
	if (!chartRef.value) {
		refList = [];
	} else {
		const first = chartRef.value?.[0];
		if (first) {
			refList = chartRef.value as unknown as HTMLDivElement[];
		} else {
			refList = [chartRef.value as unknown as HTMLDivElement];
		}
	}
	return refList;
});

const recordSetList = computed(() => props.data.filter((item) => item.type === AnswerType.RecordSet));
const summaryList = computed(() => props.data.filter((item) => item.type === AnswerType.Summary));
const urlList = computed(() => props.data.filter((item) => item.type === AnswerType.Url));

const drawAllChart = () => {
	chartInstanceList.map((item, index) => {
		drawChart(item, recordSetList.value[index]);
	});
};

const selectChartTypeChange = () => {
	drawAllChart();
};

const drawChart = (instance, data) => {
	const xType = 'time';
	let timeIndex = data.cols.findIndex((item) => item.type === 'time');
	if (timeIndex === -1) {
		timeIndex = 0;
	}
	const timeCol = data.cols[timeIndex];

	let valueIndex = data.cols.findIndex((item) => item.type === 'value');
	if (valueIndex === -1) {
		valueIndex = 2;
	}
	const valueCol = data.cols[valueIndex];

	let nameCol = null;
	let groupedValues = null;
	if (data.chart === 'muli_line') {
		let nameIndex = data.cols.findIndex((item) => item.type === 'name');
		if (nameIndex === -1) {
			nameIndex = 1;
		}
		nameCol = data.cols[nameIndex];
		groupedValues = _.groupBy(data.values, (item) => item[nameIndex]);
	} else if (data.chart === 'single_line') {
		groupedValues = {
			default: data.values,
		};
	} else {
		// 默认都当muli_line
		let nameIndex = data.cols.findIndex((item) => item.type === 'name');
		if (nameIndex === -1) {
			nameIndex = 1;
		}
		nameCol = data.cols[nameIndex];
		groupedValues = _.groupBy(data.values, (item) => item[nameIndex]);
	}

	const seriesData = Object.keys(groupedValues).map((item) => {
		const values = groupedValues[item];
		return {
			name: item === 'default' ? '' : item,
			data: values.map((item) => [item[timeIndex], item[valueIndex]]),
			type: chartTypeMapEchart[selectChartType.value],
		};
	});
	instance.setOption({
		grid: {
			// bottom: 120,
			// right: '15%',
			bottom: '5%',
		},
		toolbox: {
			show: true,
			feature: {
				dataZoom: {
					yAxisIndex: 'none',
				},

				myBar: {
					title: '转化为柱状图',
					show: true,
					icon: 'path://M580.8 228.8h-136v500.8h136V228.8z m-40 460.8h-56V268.8h56v420.8zM788.8 420.8h-136v308.8h136V420.8z m-40 268.8h-56V460.8h56v228.8zM372.8 326.4h-136v401.6h136V326.4z m-40 363.2h-56V366.4h56v323.2zM208 788.8h608v40H208z',
					onclick: () => {
						instance.setOption({
							series: seriesData.map((item) => ({
								...item,
								type: 'bar',
							})),
						});
					},
				},

				myScatter: {
					title: '转化为散点图',
					show: true,
					icon: 'path://M445.7 609.8c0 19.4 10.3 37.3 27.1 46.9 16.8 9.7 37.4 9.7 54.2 0 16.8-9.7 27.1-27.6 27.1-46.9 0-29.9-24.3-54.2-54.2-54.2s-54.2 24.3-54.2 54.2z m0 0M179.2 613.8c-42.2 0-76.5 34.3-76.5 76.5s34.3 76.5 76.5 76.5 76.5-34.3 76.5-76.5-34.3-76.5-76.5-76.5z m0 0M144.9 401.1c0 29 23.5 52.5 52.5 52.5s52.5-23.5 52.5-52.5-23.5-52.5-52.5-52.5-52.5 23.5-52.5 52.5z m0 0M598.7 404c0 42.2 34.3 76.5 76.5 76.5 42.3 0 76.5-34.3 76.5-76.5 0-42.3-34.3-76.5-76.5-76.5-42.3 0-76.5 34.3-76.5 76.5z m0 0M849.3 169.2c-42.2 0-76.5 34.3-76.5 76.5s34.3 76.5 76.5 76.5 76.5-34.3 76.5-76.5-34.3-76.5-76.5-76.5z m0 0M261.6 583.1c0 13.2 7.1 25.5 18.5 32.1 11.5 6.6 25.6 6.6 37.1 0s18.5-18.9 18.5-32.1c0-20.5-16.6-37.1-37.1-37.1-20.4 0.1-37 16.7-37 37.1z m0 0M276.8 425.1c0 42.3 34.3 76.5 76.5 76.5 42.3 0 76.5-34.3 76.5-76.5s-34.3-76.5-76.5-76.5-76.5 34.3-76.5 76.5z m0 0M445.7 421.4c0 18.5 9.9 35.5 25.8 44.8 16 9.2 35.7 9.2 51.7 0s25.8-26.3 25.8-44.8c0-28.5-23.1-51.7-51.7-51.7-28.5 0-51.6 23.2-51.6 51.7z m0 0M398.2 208.8c0 42.3 34.3 76.5 76.5 76.5s76.5-34.3 76.5-76.5c0-42.3-34.3-76.5-76.5-76.5s-76.5 34.3-76.5 76.5z m0 0M693.7 599.2c0 42.3 34.3 76.5 76.5 76.5s76.5-34.3 76.5-76.5-34.3-76.5-76.5-76.5c-42.3 0-76.5 34.3-76.5 76.5z m0 0M62.1 828.9H959v60.7H62.1z',
					onclick: () => {
						instance.setOption({
							series: seriesData.map((item) => ({
								...item,
								type: 'scatter',
							})),
						});
					},
				},
				myRestore: {
					title: '还原',
					show: true,
					icon: 'path://M909.637 478.524c-15.444-189.946-167.065-345.049-356.655-364.38-99.805-10.176-193.011 16.474-267.901 67.887l18.467-68.842c5.725-21.337-6.933-43.274-28.27-48.998s-43.274 6.934-48.998 28.271l-41.455 154.536c-5.724 21.337 6.933 43.273 28.271 48.998l154.536 41.455c21.337 5.724 43.273-6.934 48.998-28.271 5.724-21.337-6.934-43.274-28.271-48.998l-54.194-14.538c58.917-39.301 131.457-59.686 208.999-52.061 147.885 14.543 268.327 132.461 285.589 280.053 23.02 196.82-133.879 363.973-327.45 358.219-135.107-4.017-252.98-96.111-294.938-224.602-17.513-53.632-18.92-105.027-9.92-152.849 3.188-16.946-4.238-34.086-19.172-42.707l-0.049-0.028c-23.737-13.705-54.358-0.164-59.404 26.777-7.724 41.233-9.128 84.686-2.965 129.403 25.571 185.535 181.03 330.336 367.878 343.176C727.656 927.857 929.6 724.049 909.637 478.524z',
					onclick: () => {
						instance.setOption({
							series: seriesData.map((item) => ({
								...item,
								type: 'line',
							})),
						});
					},
				},
				dataView: {
					readOnly: true,
					optionToContent: timeDataOptionToContent,
				},
				saveAsImage: {},
			},
		},

		tooltip: {
			show: true,
			trigger: 'axis',
		},
		title: {
			text: data?.title,
			left: 'center',
		},
		xAxis: {
			name: timeCol?.title,
			type: xType,
		},
		yAxis: {
			name: valueCol?.title,
			type: 'value',
			axisLabel: {
				formatter: axisLabelFormatter,
			},
		},
		series: seriesData,
		dataZoom: {
			type: 'inside',
		},
	});
};

const doTableLayout = () => {
	tableRefList.value.forEach((item) => {
		item.doLayout();
	});
};
let chartInstanceList: echarts.ECharts[] = null;
onMounted(() => {
	setTimeout(() => {
		const parent = chartRefList.value[0]?.parentElement;
		const data = props.data;
		setTimeout(() => {
			doTableLayout();
		}, 3000);

		if (!parent) {
			return;
		}

		const parentBound = parent.getBoundingClientRect();
		let divideCount = 1;

		const width = parentBound.width / divideCount;
		chartInstanceList = chartRefList.value.map((item) => {
			return echarts.init(item, undefined, {
				width: width,
				height: parentBound.height,
			});
		});
		drawAllChart();
	}, 1000);
});
</script>
<style scoped lang="scss"></style>
